--[[
Spezialisierung die die Warnblinker beim Fahrzeug anschaltet, sobald dieses rckwrts fhrt.

Author:		Ifko[nator], amhh
Datum:		29.08.2016
Version:	v1.1

History:	v1.0 @ 23.08.2016 initial script, inspired by brakeLightExtension
			v1.0 @ 29.08.2016 additional code for compatibility with driveControl
			v1.1 @ 03.09.2016 added reverseDrivingBeep

This script is based on brakeLightExtension v1.8 by Ifko[nator].
]]

ReverseWarningLightsExtension = {};

function ReverseWarningLightsExtension.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization((Drivable or Attachable), specializations)
end;

function ReverseWarningLightsExtension:load(xmlFile)
	self.reverseWarningLights = {};
	local numTurnSignalsLeft = 0;
	local numTurnSignalsRight = 0;
	local numReverseLight = 0;
	
	while true do
		local parameterL = string.format("vehicle.turnSignals.turnSignalLeft(%d)", numTurnSignalsLeft);
		if not hasXMLProperty(xmlFile, parameterL) then
            break;
        end;
		
		local turnSignalsLeftOriginal = Utils.indexToObject(self.components, getXMLString(xmlFile, parameterL .. "#index"));
		if turnSignalsLeftOriginal ~= nil then
            --## simply clone the turn light. The cloned turn light will be visible, if the vehicle is moving backwards.
			local cloneTurnSignal = clone(turnSignalsLeftOriginal, true, false, false);
			setVisibility(cloneTurnSignal, false);
            table.insert(self.reverseWarningLights, cloneTurnSignal);
        end;
		
		numTurnSignalsLeft = numTurnSignalsLeft + 1;
	end;
	
	while true do
		local parameterR = string.format("vehicle.turnSignals.turnSignalRight(%d)", numTurnSignalsRight);
		if not hasXMLProperty(xmlFile, parameterR) then
            break;
        end;
		
		local turnSignalsRightOriginal = Utils.indexToObject(self.components, getXMLString(xmlFile, parameterR .. "#index"));
		if turnSignalsRightOriginal ~= nil then
            --## simply clone the turn light. The cloned turn light will be visible, if the vehicle is moving backwards.
			local cloneTurnSignal = clone(turnSignalsRightOriginal, true, false, false);
			setVisibility(cloneTurnSignal, false);
            table.insert(self.reverseWarningLights, cloneTurnSignal);
        end;
		
		numTurnSignalsRight = numTurnSignalsRight + 1;
	end;

	self.doReverse = false;
	
	-- check if verhicle has reverseDriveSound, only when vehicle is driveable
		self.hasBeepSound = true;
	if SpecializationUtil.hasSpecialization(Drivable, self.specializations) then
	
		local parameterP = "vehicle.reverseDriveSound";
		if not hasXMLProperty(xmlFile, parameterP) then
			self.hasBeepSound = false;
		end;
		
		reverseDrivingSound = "$data/vehicles/shared/reverseDrivingBeep.wav";
		reverseDrivingSound = Utils.getFilename(reverseDrivingSound, self.baseDirectory);
		--reverseDrivingSound = Utils.getFilename("reverseDrivingBeep.wav", self.baseDirectory);
		--reverseDrivingSound = Utils.getFilename("reverseDrivingBeep.wav", g_currentModDirectory);
		self.reverseDrivingSoundSample = createSample("reverseBeep");
		loadSample(self.reverseDrivingSoundSample, reverseDrivingSound, false);
		
	else
		self.hasBeepSound = true;
	end;
	self.isBeeping = false;

end;


function ReverseWarningLightsExtension:update(dt)
	for _, cloneTurnSignal in pairs(self.reverseWarningLights) do
		if self:getIsActive() then
			if SpecializationUtil.hasSpecialization(Drivable, self.specializations) then
				local movingDirection;
				movingDirection = self.movingDirection;
							
				--[[ alternate method to get movingDirection
				if self.isEntered then
					movingDirection = InputBinding.getDigitalInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE);
					
					if InputBinding.isAxisZero(movingDirection) then
						--## controller
						movingDirection = InputBinding.getAnalogInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE);
					end;
				else
					movingDirection = 0;
				end;]]
				
				-- wheelSpeed as an alternate option
				--local wheelSpeed = getWheelShapeAxleSpeed(self.wheels[1].node, self.wheels[1].wheelShape);
				--if wheelSpeed < -0.15 then
					
				local someoneIsInTheVehicle; 
				
				if self.cp ~= nil and self.modFM == nil then
					someoneIsInTheVehicle = self.cp.isDriving or self.isEntered or self.isControlled or self.isHired;
				elseif self.cp ~= nil and self.modFM ~= nil then
					someoneIsInTheVehicle = self.cp.isDriving or self.modFM.FollowState ~= FollowMe.STATE_NONE or self.isEntered or self.isControlled or self.isHired;
				elseif self.cp == nil and self.modFM ~= nil then
					someoneIsInTheVehicle = self.modFM.FollowState ~= FollowMe.STATE_NONE or self.isEntered or self.isControlled or self.isHired;
				else
					someoneIsInTheVehicle = self.isEntered or self.isControlled or self.isHired; 
				end;
				
				local cruiseControlIsOff;
				
				if self.isHired then
					if self.isHirableBlocked then
						if self.cruiseControl.state == Drivable.CRUISECONTROL_STATE_OFF then
							cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_OFF;
						else
							cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_ACTIVE;
						end;
					else
						cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_OFF;
					end;
				elseif self.cp ~= nil and self.cp.isDriving then
					if not self.cp.TrafficBrake then
						if self.cruiseControl.state == Drivable.CRUISECONTROL_STATE_OFF then
							cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_OFF;
						else
							cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_ACTIVE;
						end;
					else
						cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_OFF;
					end;
				else
					cruiseControlIsOff = Drivable.CRUISECONTROL_STATE_OFF;
				end;

				--if movingDirection < -0.25 and self.isMotorStarted==true and someoneIsInTheVehicle and self.cruiseControl.state == cruiseControlIsOff then
				if someoneIsInTheVehicle and movingDirection < -0.25 and self.isMotorStarted and self.cruiseControl.state == cruiseControlIsOff or self.isMotorStarted and movingDirection < -0.15 and self.cruiseControl.state == cruiseControlIsOff then
					setVisibility(cloneTurnSignal, true);
					self.doReverse = true;
					if self.hasBeepSound==false and not self.isBeeping then
						playSample(self.reverseDrivingSoundSample,0,0.5,0);
						self.isBeeping = true;
					end;
				else
					setVisibility(cloneTurnSignal, false);
					self.doReverse = false;
					if self.isBeeping then
						stopSample(self.reverseDrivingSoundSample);
						self.isBeeping = false;
					end;
				end;

			elseif SpecializationUtil.hasSpecialization(Attachable, self.specializations) then
				--## attachable vehicle
				if self.attacherVehicle ~= nil and self.attacherVehicle:getIsActive() and self.attacherVehicle.doReverse then
					setVisibility(cloneTurnSignal, true);
					self.doReverse = true;
				else
					setVisibility(cloneTurnSignal, false);
					self.doReverse = false;
				end;
			end;
		else
			setVisibility(cloneTurnSignal, false);
			--self.doReverse = false;
		end;
	end;
	
	--## support for all attachables, wich have no brake lights, for example: weights.
	if SpecializationUtil.hasSpecialization(Attachable, self.specializations) then
		if #self.reverseWarningLights == 0 then
			if self.attacherVehicle ~= nil and self.attacherVehicle:getIsActive() and self.attacherVehicle.doReverse then
				self.doReverse = true;
			else
				self.doReverse = false;
			end;
		end;
	end;
end;

function ReverseWarningLightsExtension:draw()end;
function ReverseWarningLightsExtension:delete()
	if self.reverseDrivingSoundSample then
		delete(self.reverseDrivingSoundSample);
	end;
end;
function ReverseWarningLightsExtension:mouseEvent(posX, posY, isDown, isUp, button)end;
function ReverseWarningLightsExtension:keyEvent(unicode, sym, modifier, isDown)end;
function ReverseWarningLightsExtension:onLeave()
	if not self.isMotorStarted and self.isBeeping then
		stopSample(self.reverseDrivingSoundSample);
	end;
end;